################################################################################################
##                                                                                            ##
##              Functions used to compute the susceptibility in the Kondo model               ##
##                                                                                            ##
################################################################################################

# generate a single flow equation (read from stdin)
function flow_eq {
  dim="$1"
  magfactor="$2"

  # read input
  pre=""
  while read line; do
    pre="$pre
$line"
  done

  # generate the Kondo configuration file
  config=$(echo "$pre" | "$meankondo_bin"/kondo_preprocess -d "$dim")

  # strip int file
  echo "$config" | strip_config "#!symbols #!groups #!propagator #!input_polynomial #!id_table #!initial_condition #!labels #!fields #!identities"

  # write the labels
  echo "#!labels"
  echo "$config" | find_config_entry "#!labels"
  # add mag and susc
  echo ","
  echo "$magindex:\"mag\","
  echo "$suscindex:\"susc\""
  echo "&"

  # add flow equation
  echo '#!flow_equation'
  # add magnetization and susceptibility
  echo "[%$magindex] = ($((2**magfactor)))[%$magindex] + [dC1][/C1^1]"
  echo ","
  echo "[%$suscindex] = ($((2**magfactor)))[%$suscindex] + [C1][ddC1][/C1^2] + [dC1][dC1][/C1^2]"
  # int flow equation
  echo ","
  echo "$config" | "$meankondo_bin"/meankondo -t "$threads" | "$meankondo_bin"/meantools derive -d 2
}

# combine an int and a noint flow equation
function combine_int_noint {
  int="$1"
  noint="$2"
  dim="$3"

  # compute flow equation

  # generate the Kondo configuration
  int=$("$meankondo_bin"/kondo_preprocess -d "$dim" "$int")
  noint=$("$meankondo_bin"/kondo_preprocess -d "$dim" "$noint")

  # strip int file
  echo "$int" | strip_config "#!symbols #!groups #!propagator #!input_polynomial #!id_table #!initial_condition #!labels #!fields #!identities"

  # write the labels
  echo "#!labels"
  echo "$int" | find_config_entry "#!labels"
  # append the labels of noint
  echo ","
  echo "$noint" | find_config_entry "#!labels"
  # add mag and susc
  echo ","
  echo "$magindex:\"mag\","
  echo "$suscindex:\"susc\""
  echo "&"

  # add flow equation
  echo '#!flow_equation'
  # add magnetization and susceptibility
  echo "[%$magindex] = ($((2**(dim-1))))[%$magindex] + [dC1][/C1^1] + (-1)[dC2][/C2^1]"
  echo ","
  echo "[%$suscindex] = ($((2**(dim-1))))[%$suscindex] + [C1][ddC1][/C1^2] + [dC1][dC1][/C1^2] + (-1)[C2][ddC2][/C2^2] + (-1)[dC2][dC2][/C2^1]"
  # int flow equation
  echo ","
  echo "$int" | "$meankondo_bin"/meankondo -t "$threads" | "$meankondo_bin"/meantools derive -d 2 -V "0,1,2,3,11,12,13,14,25,31,101,111,-1"
  # noint flow equation
  echo ","
  # change C1 into C2
  echo "$noint" | "$meankondo_bin"/meankondo -t "$threads" | "$meankondo_bin"/meantools derive -d 2 | sed 's/C1/C2/g'
}


# initial conditions: expand exponential
function exp_init_cd {
  while read line; do
    pre="$pre
$line"
  done
  init="$1"
  dim="$2"
  vars="$3"
  h="$4"

  initcd=$(echo "$init" | sed -r 's/ //g; s/h/'"$h"'/g')

  echo "$pre" | "$meankondo_bin"/kondo_preprocess -d "$dim" | "$meankondo_bin"/meantools exp | "$meankondo_bin"/meantools derive -d 2 -V "$vars" | "$meankondo_bin"/meantools eval -R "$initcd" 
}
